@@ -24,12 +24,15 @@
 #include "dynload.h"
 #include "tilib.h"
 #include "tilib_defs.h"
-#include "threads.h"
+#include "thread.h"
+#include "ctrlc.h"
 
-#ifdef WIN32
+#if defined(__Windows__) || defined(__CYGWIN__)
 static const char tilib_filename[] = "MSP430.DLL";
+#define TIDLL __stdcall
 #else
 static const char tilib_filename[] = "libmsp430.so";
+#define TIDLL
 #endif
 
 struct tilib_device {
@@ -37,45 +40,51 @@ struct tilib_device {
 
 	dynload_handle_t	hnd;
 
-	threads_lock_t		mb_lock;
+	thread_lock_t		mb_lock;
 	uint32_t		mailbox;
 
 	uint16_t		bp_handles[DEVICE_MAX_BREAKPOINTS];
 
+	char			uifPath[1024];
+
 	/* MSP430.h */
-	STATUS_T WINAPI (*MSP430_Initialize)(char *port, long *version);
-	STATUS_T WINAPI (*MSP430_VCC)(long voltage);
-	STATUS_T WINAPI (*MSP430_Configure)(long mode, long value);
-	STATUS_T WINAPI (*MSP430_OpenDevice)(char *Device, char *Password,
+	STATUS_T TIDLL (*MSP430_Initialize)(char *port, long *version);
+	STATUS_T TIDLL (*MSP430_VCC)(long voltage);
+	STATUS_T TIDLL (*MSP430_Configure)(long mode, long value);
+	STATUS_T TIDLL (*MSP430_OpenDevice)(char *Device, char *Password,
 					     long PwLength, long DeviceCode,
 					     long setId);
-	STATUS_T WINAPI (*MSP430_GetFoundDevice)(char *FoundDevice,
+	STATUS_T TIDLL (*MSP430_GetFoundDevice)(char *FoundDevice,
 						 long count);
-	STATUS_T WINAPI (*MSP430_Close)(long vccOff);
-	STATUS_T WINAPI (*MSP430_Memory)(long address, char *buffer,
+	STATUS_T TIDLL (*MSP430_Close)(long vccOff);
+	STATUS_T TIDLL (*MSP430_Memory)(long address, char *buffer,
 					 long count, long rw);
-	STATUS_T WINAPI (*MSP430_Reset)(long method, long execute,
+	STATUS_T TIDLL (*MSP430_Reset)(long method, long execute,
 					long releaseJTAG);
-	STATUS_T WINAPI (*MSP430_Erase)(long type, long address, long length);
-	STATUS_T WINAPI (*MSP430_Error_Number)(void);
-	const char * WINAPI (*MSP430_Error_String)(long errNumber);
+	STATUS_T TIDLL (*MSP430_Erase)(long type, long address, long length);
+	STATUS_T TIDLL (*MSP430_Error_Number)(void);
+	const char *TIDLL (*MSP430_Error_String)(long errNumber);
+
+	STATUS_T TIDLL (*MSP430_GetNumberOfUsbIfs)(long* number);
+	STATUS_T TIDLL (*MSP430_GetNameOfUsbIf)(long idx, char **name,
+						 long *status);
 
 	/* MSP430_Debug.h */
-	STATUS_T WINAPI (*MSP430_Registers)(long *registers, long mask,
+	STATUS_T TIDLL (*MSP430_Registers)(long *registers, long mask,
 					    long rw);
-	STATUS_T WINAPI (*MSP430_Run)(long mode, long releaseJTAG);
-        STATUS_T WINAPI (*MSP430_State)(long* state, long stop,
-					long* pCPUCycles);
+	STATUS_T TIDLL (*MSP430_Run)(long mode, long releaseJTAG);
+	STATUS_T TIDLL (*MSP430_State)(long *state, long stop,
+					long *pCPUCycles);
 
 	/* MSP430_EEM.h */
-	STATUS_T WINAPI (*MSP430_EEM_Init)(DLL430_EVENTNOTIFY_FUNC callback,
+	STATUS_T TIDLL (*MSP430_EEM_Init)(DLL430_EVENTNOTIFY_FUNC callback,
 					   long clientHandle,
 					   MessageID_t *pMsgIdBuffer);
-	STATUS_T WINAPI (*MSP430_EEM_SetBreakpoint)(uint16_t *pwBpHandle,
+	STATUS_T TIDLL (*MSP430_EEM_SetBreakpoint)(uint16_t *pwBpHandle,
 						    BpParameter_t *pBpBuffer);
 
 	/* MSP430_FET.h */
-	STATUS_T WINAPI (*MSP430_FET_FwUpdate)(char* lpszFileName,
+	STATUS_T TIDLL (*MSP430_FET_FwUpdate)(char* lpszFileName,
 					   DLL430_FET_NOTIFY_FUNC callback,
 					   long clientHandle);
 
@@ -107,19 +116,19 @@ static void event_notify(unsigned int msg_id, unsigned int w_param,
 	(void)w_param;
 	(void)l_param;
 
-	threads_lock_acquire(&dev->mb_lock);
+	thread_lock_acquire(&dev->mb_lock);
 	dev->mailbox |= msg_id;
-	threads_lock_release(&dev->mb_lock);
+	thread_lock_release(&dev->mb_lock);
 }
 
 static uint32_t event_fetch(struct tilib_device *dev)
 {
 	uint32_t ret;
 
-	threads_lock_acquire(&dev->mb_lock);
+	thread_lock_acquire(&dev->mb_lock);
 	ret = dev->mailbox;
 	dev->mailbox = 0;
-	threads_lock_release(&dev->mb_lock);
+	thread_lock_release(&dev->mb_lock);
 
 	return ret;
 }
@@ -184,6 +193,16 @@ static int get_all_funcs(struct tilib_device *dev)
 	if (!dev->MSP430_Error_String)
 		return -1;
 
+	dev->MSP430_GetNumberOfUsbIfs =
+		get_func(dev->hnd, "MSP430_GetNumberOfUsbIfs");
+	if (!dev->MSP430_GetNumberOfUsbIfs)
+		return -1;
+
+	dev->MSP430_GetNameOfUsbIf =
+		get_func(dev->hnd, "MSP430_GetNameOfUsbIf");
+	if (!dev->MSP430_GetNameOfUsbIf)
+		return -1;
+
 	dev->MSP430_Registers = get_func(dev->hnd, "MSP430_Registers");
 	if (!dev->MSP430_Registers)
 		return -1;
@@ -217,7 +236,7 @@ static void report_error(struct tilib_device *dev, const char *what)
 	long err = dev->MSP430_Error_Number();
 	const char *desc = dev->MSP430_Error_String(err);
 
-	printc_err("tilib: %s: %s (error = %d)\n", what, desc, err);
+	printc_err("tilib: %s: %s (error = %ld)\n", what, desc, err);
 }
 
 static int tilib_readmem(device_t dev_base, address_t addr,
@@ -262,6 +281,9 @@ static int tilib_erase(device_t dev_base, device_erase_type_t type,
 {
 	struct tilib_device *dev = (struct tilib_device *)dev_base;
 
+	if (type == DEVICE_ERASE_MAIN)
+		address = 0xfffe;
+
 	if (dev->MSP430_Erase(ti_erase_type(type), address, 0) < 0) {
 		report_error(dev, "MSP430_Erase");
 		return -1;
@@ -304,6 +326,47 @@ static int tilib_setregs(device_t dev_base, const address_t *regs)
 	return 0;
 }
 
+static void load_break(BpParameter_t *param, address_t addr)
+{
+	param->bpMode = BP_CODE;
+	param->lAddrVal = addr;
+	param->bpType = BP_MAB;
+	param->lReg = 0; /* not used */
+	param->bpAccess = BP_FETCH;
+	param->bpAction = BP_BRK;
+	param->bpOperat = BP_EQUAL;
+	param->lMask = 0; /* what's this? */
+	param->lRangeEndAdVa = 0; /* not used */
+	param->bpRangeAction = 0; /* not used */
+	param->bpCondition = BP_NO_COND;
+	param->lCondMdbVal = 0;
+	param->bpCondAccess = BP_FETCH;
+	param->lCondMask = 0; /* what's this? */
+	param->bpCondOperat = BP_EQUAL;
+	param->wExtCombine = 0; /* not used? */
+}
+
+static void load_complex(BpParameter_t *param, address_t addr,
+			 BpAccess_t acc)
+{
+	param->bpMode = BP_COMPLEX;
+	param->lAddrVal = addr;
+	param->bpType = BP_MAB;
+	param->lReg = 0; /* not used (only for register-write) */
+	param->bpAccess = acc;
+	param->bpAction = BP_BRK;
+	param->bpOperat = BP_EQUAL;
+	param->lMask = 0xffffff;
+	param->lRangeEndAdVa = 0; /* not used */
+	param->bpRangeAction = 0; /* not used */
+	param->bpCondition = BP_NO_COND;
+	param->lCondMdbVal = 0;
+	param->bpCondAccess = acc;
+	param->lCondMask = 0; /* what's this? */
+	param->bpCondOperat = BP_EQUAL;
+	param->wExtCombine = 0; /* not used? */
+}
+
 static int refresh_bps(struct tilib_device *dev)
 {
 	int i;
@@ -316,22 +379,28 @@ static int refresh_bps(struct tilib_device *dev)
 			continue;
 
 		if (bp->flags & DEVICE_BP_ENABLED) {
-			param.bpMode = BP_CODE;
-			param.lAddrVal = bp->addr;
-			param.bpType = BP_MAB;
-			param.lReg = 0; /* not used */
-			param.bpAccess = BP_FETCH;
-			param.bpAction = BP_BRK;
-			param.bpOperat = BP_EQUAL;
-			param.lMask = 0; /* what's this? */
-			param.lRangeEndAdVa = 0; /* not used */
-			param.bpRangeAction = 0; /* not used */
-			param.bpCondition = BP_NO_COND;
-			param.lCondMdbVal = 0;
-			param.bpCondAccess = BP_FETCH;
-			param.lCondMask = 0; /* what's this? */
-			param.bpCondOperat = BP_EQUAL;
-			param.wExtCombine = 0; /* not used? */
+			switch (bp->type) {
+			case DEVICE_BPTYPE_BREAK:
+				load_break(&param, bp->addr);
+				break;
+
+			case DEVICE_BPTYPE_WATCH:
+				load_complex(&param, bp->addr, BP_NO_FETCH);
+				break;
+
+			case DEVICE_BPTYPE_READ:
+				load_complex(&param, bp->addr,
+					     BP_READ_DMA);
+				break;
+
+			case DEVICE_BPTYPE_WRITE:
+				load_complex(&param, bp->addr,
+					     BP_WRITE_DMA);
+				break;
+			}
+		} else if (!dev->bp_handles[i]) {
+			bp->flags &= ~DEVICE_BP_DIRTY;
+			continue;
 		} else {
 			param.bpMode = BP_CLEAR;
 		}
@@ -408,8 +477,7 @@ static device_status_t tilib_poll(device_t dev_base)
 {
 	struct tilib_device *dev = (struct tilib_device *)dev_base;
 
-        ctrlc_reset();
-        if ((usleep(50000) < 0) || ctrlc_check())
+        if ((delay_ms(50) < 0) || ctrlc_check())
                 return DEVICE_STATUS_INTR;
 
 	if (event_fetch(dev) & MID_HALT_ANY)
@@ -429,7 +497,7 @@ static void tilib_destroy(device_t dev_base)
 	printc_dbg("MSP430_Close\n");
 	dev->MSP430_Close(0);
 	dynload_close(dev->hnd);
-	threads_lock_destroy(&dev->mb_lock);
+	thread_lock_destroy(&dev->mb_lock);
 	free(dev);
 }
 
@@ -445,7 +513,7 @@ static void fw_progress(unsigned int msg_id, unsigned long w_param,
 		if (w_param > 100)
 			w_param = 100;
 
-		printc("   %3d percent done\n", w_param);
+		printc("   %3lu percent done\n", w_param);
 		break;
 
 	case BL_UPDATE_ERROR:
@@ -497,17 +565,10 @@ static int do_fw_update(struct tilib_device *dev, const char *filename)
 static int do_init(struct tilib_device *dev, const struct device_args *args)
 {
 	long version;
-	char buf[1024];
 	union DEVICE_T device;
 
-	/* Not sure if the path is actually modified by MSP430_Initialize,
-	 * but the argument isn't const, so probably safest to copy it.
-	 */
-	strncpy(buf, args->path, sizeof(buf));
-	buf[sizeof(buf) - 1] = 0;
-
-	printc_dbg("MSP430_Initialize: %s\n", buf);
-	if (dev->MSP430_Initialize(buf, &version) < 0) {
+	printc_dbg("MSP430_Initialize: %s\n", dev->uifPath);
+	if (dev->MSP430_Initialize(dev->uifPath, &version) < 0) {
 		report_error(dev, "MSP430_Initialize");
 		return -1;
 	}
@@ -535,7 +596,7 @@ static int do_init(struct tilib_device *dev, const struct device_args *args)
 			return -1;
 		}
 	} else {
-		printc_dbg("Firmware version is %d\n", version);
+		printc_dbg("Firmware version is %ld\n", version);
 	}
 
 	printc_dbg("MSP430_VCC: %d mV\n", args->vcc_mv);
@@ -546,7 +607,7 @@ static int do_init(struct tilib_device *dev, const struct device_args *args)
 	}
 
 	/* Without this delay, MSP430_OpenDevice will often hang. */
-	usleep(1000000);
+	delay_s(1);
 
 	printc_dbg("MSP430_OpenDevice\n");
 	if (dev->MSP430_OpenDevice("DEVICE_UNKNOWN", "", 0, 0, 0) < 0) {
@@ -570,27 +631,59 @@ static int do_init(struct tilib_device *dev, const struct device_args *args)
 		dev->base.max_breakpoints = DEVICE_MAX_BREAKPOINTS;
 
 	printc_dbg("MSP430_EEM_Init\n");
-	threads_lock_init(&dev->mb_lock);
+	thread_lock_init(&dev->mb_lock);
 	if (dev->MSP430_EEM_Init(event_notify, (long)dev,
 				 (MessageID_t *)&my_message_ids) < 0) {
 		report_error(dev, "MSP430_EEM_Init");
 		dev->MSP430_Close(0);
-		threads_lock_destroy(&dev->mb_lock);
+		thread_lock_destroy(&dev->mb_lock);
 		return -1;
 	}
 
 	return 0;
 }
 
+static int do_findUif(struct tilib_device *dev)
+{
+	// Find the first uif and store the path name into dev->uifPath
+	long attachedUifCount = 0;
+	long uifIndex = 0;
+
+	printc_dbg("MSP430_GetNumberOfUsbIfs\n");
+	if (dev->MSP430_GetNumberOfUsbIfs(&attachedUifCount) < 0) {
+		report_error(dev, "MSP430_GetNumberOfUsbIfs");
+		return -1;
+	}
+
+	for (uifIndex = 0; uifIndex < attachedUifCount; uifIndex++)
+	{
+		char *name = NULL;
+		long status = 0;
+
+		printc_dbg("MSP430_GetNameOfUsbIf\n");
+		if (dev->MSP430_GetNameOfUsbIf(uifIndex, &name, &status) < 0) {
+			report_error(dev, "MSP430_GetNameOfUsbIf");
+			return -1;
+		}
+
+		if (status == 0) /* status == 1 when fet is in use */
+		{
+			// This fet is unused
+			strncpy(dev->uifPath, name, sizeof(dev->uifPath));
+			printc_dbg("Found FET: %s\n", dev->uifPath);
+			return 0;
+		}
+	}
+
+	printc_err("No unused FET found.\n");
+
+	return -1;
+}
+
 static device_t tilib_open(const struct device_args *args)
 {
 	struct tilib_device *dev;
 
-	if (!(args->flags & DEVICE_FLAG_TTY)) {
-		printc_err("This driver does not support raw USB access.\n");
-		return NULL;
-	}
-
 	dev = malloc(sizeof(*dev));
 	if (!dev) {
 		printc_err("tilib: can't allocate memory: %s\n",
@@ -615,6 +708,24 @@ static device_t tilib_open(const struct device_args *args)
 		return NULL;
 	}
 
+	/* Copy the args->path to the dev->uifPath buffer
+	 * we may need to change it for automatic detection, and
+	 * not sure if the path is actually modified by MSP430_Initialize,
+	 * but the argument isn't const, so probably safest to copy it.
+	 */
+
+	if ((args->flags & DEVICE_FLAG_TTY)) {
+		strncpy(dev->uifPath, args->path, sizeof(dev->uifPath));
+		dev->uifPath[sizeof(dev->uifPath) - 1] = 0;
+	} else {
+		// No path was supplied, use the first UIF we can find
+		if (do_findUif(dev) < 0) {
+			dynload_close(dev->hnd);
+			free(dev);
+			return NULL;
+		}
+	}
+
 	if (do_init(dev, args) < 0) {
 		printc_err("tilib: device initialization failed\n");
 		dynload_close(dev->hnd);
