@@ -31,9 +31,14 @@ int cmd_dis(char **arg);
 int cmd_hexout(char **arg);
 int cmd_prog(char **arg);
 int cmd_load(char **arg);
+int cmd_verify(char **arg);
 int cmd_setbreak(char **arg);
+int cmd_setwatch(char **arg);
+int cmd_setwatch_r(char **arg);
+int cmd_setwatch_w(char **arg);
 int cmd_delbreak(char **arg);
 int cmd_break(char **arg);
 int cmd_locka(char **arg);
+int cmd_fill(char **arg);
 
 #endif
