@@ -33,6 +33,7 @@
 #include "reader.h"
 #include "expr.h"
 #include "gdb_proto.h"
+#include "ctrlc.h"
 
 /************************************************************************
  * GDB server
@@ -309,8 +310,8 @@ static int run(struct gdb_data *data, char *buf)
 static int set_breakpoint(struct gdb_data *data, int enable, char *buf)
 {
 	char *parts[2];
-	int type;
 	address_t addr;
+	device_bptype_t type;
 	int i;
 
 	/* Break up the arguments */
@@ -323,9 +324,25 @@ static int set_breakpoint(struct gdb_data *data, int enable, char *buf)
 		return gdb_send(data, "E00");
 	}
 
-	/* We only support breakpoints */
-	type = atoi(parts[0]);
-	if (type < 0 || type > 1) {
+	switch (atoi(parts[0])) {
+	case 0:
+	case 1:
+		type = DEVICE_BPTYPE_BREAK;
+		break;
+
+	case 2:
+		type = DEVICE_BPTYPE_WRITE;
+		break;
+
+	case 3:
+		type = DEVICE_BPTYPE_READ;
+		break;
+
+	case 4:
+		type = DEVICE_BPTYPE_WATCH;
+		break;
+
+	default:
 		printc_err("gdb: unsupported breakpoint type: %s\n",
 			parts[0]);
 		return gdb_send(data, "");
@@ -341,7 +358,7 @@ static int set_breakpoint(struct gdb_data *data, int enable, char *buf)
 	addr = strtoul(parts[1], NULL, 16);
 
 	if (enable) {
-		if (device_setbrk(device_default, -1, 1, addr) < 0) {
+		if (device_setbrk(device_default, -1, 1, addr, type) < 0) {
 			printc_err("gdb: can't add breakpoint at "
 				"0x%04x\n", addr);
 			return gdb_send(data, "E00");
@@ -349,7 +366,7 @@ static int set_breakpoint(struct gdb_data *data, int enable, char *buf)
 
 		printc("Breakpoint set at 0x%04x\n", addr);
 	} else {
-		device_setbrk(device_default, -1, 0, addr);
+		device_setbrk(device_default, -1, 0, addr, type);
 		printc("Breakpoint cleared at 0x%04x\n", addr);
 	}
 
@@ -434,7 +451,7 @@ static int process_gdb_command(struct gdb_data *data, char *buf)
 
 static void gdb_reader_loop(struct gdb_data *data)
 {
-	for (;;) {
+	while (!ctrlc_check()) {
 		char buf[GDB_BUF_SIZE];
 		int len = 0;
 
@@ -502,7 +519,7 @@ static int gdb_server(int port)
 	/* Put the hardware breakpoint setting into a known state. */
 	printc("Clearing all breakpoints...\n");
 	for (i = 0; i < device_default->max_breakpoints; i++)
-		device_setbrk(device_default, i, 0, 0);
+		device_setbrk(device_default, i, 0, 0, 0);
 
 #ifdef DEBUG_GDB
 	printc("starting GDB reader loop...\n");
