@@ -25,6 +25,7 @@
 #include "vector.h"
 #include "stdcmd.h"
 #include "output.h"
+#include "output_util.h"
 #include "reader.h"
 #include "expr.h"
 
@@ -105,7 +106,7 @@ int cmd_help(char **arg)
 		vector_destroy(&v);
 
 		printc("Type \"help <topic>\" for more information.\n");
-#if defined(WIN32) && !defined(USE_READLINE)
+#if defined(__Windows__) && !defined(USE_READLINE)
 		printc("Press Ctrl+Z, Enter to quit.\n");
 #else
 		printc("Press Ctrl+D to quit.\n");
