@@ -18,15 +18,18 @@
 
 #include <stdio.h>
 #include <assert.h>
+#include <string.h>
+#include <stdlib.h>
 
 #include "dis.h"
 #include "output_util.h"
 #include "stab.h"
 #include "util.h"
+#include "demangle.h"
 
-static int format_addr(msp430_amode_t amode, uint16_t addr)
+static int format_addr(msp430_amode_t amode, address_t addr)
 {
-	char name[64];
+	char name[MAX_SYMBOL_LENGTH];
 	const char *prefix = "";
 
 	switch (amode) {
@@ -91,7 +94,7 @@ static int format_reg(msp430_amode_t amode, msp430_reg_t reg)
  *
  * Returns the number of characters printed.
  */
-static int format_operand(msp430_amode_t amode, uint16_t addr,
+static int format_operand(msp430_amode_t amode, address_t addr,
 			  msp430_reg_t reg)
 {
 	int len = 0;
@@ -135,10 +138,10 @@ static int dis_format(const struct msp430_instruction *insn)
 				      insn->src_addr,
 				      insn->src_reg);
 
-		printc(",");
+		len += printc(",");
 		while (len < 15)
 			len += printc(" ");
-		printc(" ");
+		len += printc(" ");
 	}
 
 	/* Destination operand */
@@ -156,9 +159,12 @@ static int dis_format(const struct msp430_instruction *insn)
 	return len;
 }
 
-void disassemble(address_t offset, const uint8_t *data, int length)
+void disassemble(address_t offset, const uint8_t *data, int length,
+		 powerbuf_t power)
 {
 	int first_line = 1;
+	unsigned long long ua_total = 0;
+	int samples_total = 0;
 
 	while (length) {
 		struct msp430_instruction insn = {0};
@@ -166,14 +172,14 @@ void disassemble(address_t offset, const uint8_t *data, int length)
 		int count;
 		int i;
 		address_t oboff;
-		char obname[64];
+		char obname[MAX_SYMBOL_LENGTH];
 
-		if (!stab_nearest(offset, obname, sizeof(obname), &oboff) &&
-		    !oboff) {
-			printc("\x1b[m%s\x1b[0m:\n", obname);
-		} else if (first_line) {
-			print_address(offset, obname, sizeof(obname));
-			printc("\x1b[m%s\x1b[0m:\n", obname);
+		if (first_line ||
+			(!stab_nearest(offset, obname, sizeof(obname), &oboff) &&
+			 !oboff)) {
+			char buffer[MAX_SYMBOL_LENGTH];
+			print_address(offset, buffer, sizeof(buffer));
+			printc("\x1b[m%s\x1b[0m:\n", buffer);
 		}
 		first_line = 0;
 
@@ -193,13 +199,39 @@ void disassemble(address_t offset, const uint8_t *data, int length)
 		}
 
 		if (retval >= 0)
-			dis_format(&insn);
+			i = dis_format(&insn);
+
+		if (power) {
+			unsigned long long ua;
+			int samples;
+
+			while (i < 40) {
+				printc(" ");
+				i++;
+			}
+
+			samples = powerbuf_get_by_mab(power, offset, &ua);
+			if (samples) {
+				printc(" ;; %.01f uA",
+					(double)ua / (double)samples);
+				ua_total += ua;
+				samples_total += samples;
+			}
+		}
+
 		printc("\n");
 
 		offset += count;
 		length -= count;
 		data += count;
 	}
+
+	if (power && samples_total)
+		printc(";; Total over this block: "
+			"%.01f uAs in %.01f ms (%.01f uA avg)\n",
+		       (double)(ua_total * power->interval_us) / 1000000.0,
+		       (double)(samples_total * power->interval_us) / 1000.0,
+		       (double)ua_total / (double)samples_total);
 }
 
 void hexdump(address_t addr, const uint8_t *data, int data_len)
@@ -254,18 +286,73 @@ void show_regs(const address_t *regs)
 
 int print_address(address_t addr, char *out, int max_len)
 {
-	char name[128];
-        address_t offset;
+	char name[MAX_SYMBOL_LENGTH];
+	address_t offset;
 
-        if (!stab_nearest(addr, name, sizeof(name), &offset)) {
+	if (!stab_nearest(addr, name, sizeof(name), &offset)) {
+		int len;
 		if (offset)
-			snprintf(out, max_len, "%s+0x%x", name, offset);
+			len = snprintf(out, max_len, "%s+0x%x", name, offset);
 		else
-			snprintf(out, max_len, "%s", name);
+			len = snprintf(out, max_len, "%s", name);
 
+		char demangled[MAX_SYMBOL_LENGTH];
+		if (demangle(name, demangled, sizeof(demangled)) > 0) {
+			snprintf(out + len, max_len - len, " (%s)", demangled);
+		}
 		return 1;
-        }
+	}
 
 	snprintf(out, max_len, "0x%04x", addr);
 	return 0;
 }
+
+/************************************************************************
+ * Name lists
+ */
+
+static int namelist_cmp(const void *a, const void *b)
+{
+	return strcasecmp(*(const char **)a, *(const char **)b);
+}
+
+void namelist_print(struct vector *v)
+{
+	int i;
+	int max_len = 0;
+	int rows, cols;
+
+	qsort(v->ptr, v->size, v->elemsize, namelist_cmp);
+
+	for (i = 0; i < v->size; i++) {
+		const char *text = VECTOR_AT(*v, i, const char *);
+		int len = strlen(text);
+
+		if (len > max_len)
+			max_len = len;
+	}
+
+	max_len += 2;
+	cols = 72 / max_len;
+	rows = (v->size + cols - 1) / cols;
+
+	for (i = 0; i < rows; i++) {
+		int j;
+
+		printc("    ");
+		for (j = 0; j < cols; j++) {
+			int k = j * rows + i;
+			const char *text;
+
+			if (k >= v->size)
+				break;
+
+			text = VECTOR_AT(*v, k, const char *);
+			printc("%s", text);
+			for (k = strlen(text); k < max_len; k++)
+				printc(" ");
+		}
+
+		printc("\n");
+	}
+}
